/**
 * Created by pawanpawar on 1/6/17.
 */
'use strict';

var extend = require('gextend');
var spawn = require('child_process').spawn;
var suspawn = require('suspawn');
var async = require('async');
var os = require('os');
var interfaces = os.networkInterfaces();
var console = new (require('my-console')).Console();

interfaces = Object.keys(interfaces);

var DEFAULTS = {
    command: 'arp-scan',
    args: ['-l'],
    interface: 'wlan0',
    listByInterface: false,
    sudo: false
};

var scan = function (cb, options) {

    var buffer = '',
        out = [],
        errbuf = '',
        arp = {};

    options.args = options.args.concat(['--ignoredups', '-interface', options.interface]);

    var cmd = options.command + ' ' + options.args.join(' ');
    if (options.verbose) {
        console.log(options.sudo ? 'sudo ' : '' + cmd);
    }
    // console.info(options.command, options.args);
    if (options.sudo) {
        arp = suspawn(options.command, options.args);
    } else {
        arp = spawn(options.command, options.args);
    }

    if (arp && arp.stdout) {
        arp.stdout.on('data', function onData (data) {
            buffer += data;
        });
        arp.stderr.on('data', function onError (data) {
            errbuf += data;
        });

        arp.on('close', function onClose (code) {
            if (code !== 0) return cb(code, null);

            buffer = buffer.split('\n');
            buffer = buffer.slice(2, -4);

            buffer.forEach(function lineParser (line) {
                var chunks = line.split('\t');
                out.push({
                    ip: chunks[0],
                    mac: (chunks[1] || '').toUpperCase(),
                    vendor: chunks[2],
                    //interfaces: interfaces,
                    timestamp: Date.now()
                });
            });
            cb(null, out);
        });

        arp.on('error', function (err) {
            cb(err, null);
        });

        arp.on('exit', function (code, signal) {

        });
    } else {
        cb(true, null);
    }
};

function scanner (callback, options) {
    var list = [];

    if (options !== undefined && (options.interface !== undefined && options.interface !== '')) {
        options = extend({}, DEFAULTS, options);
        scan(function (err, data) {
            if (err) {
                // console.log('Error in ' + options.interface + ' interface. Error code is ' + err);
            } else {
                if (options.listByInterface) {
                    var result = {};
                    result[options.interface] = data;
                    list.push(result);
                } else {
                    if (data && data.length) {
                        for (var kk = 0; kk < data.length; kk++) {
                            list.push(data[kk]);
                        }
                    }
                }
            }
            callback(err, list)
        }, options);
    } else {
        options = extend({}, DEFAULTS, options);
        async.eachSeries(interfaces, function (inter, cbl) {
            if (inter !== 'lo') {
                options.args = ['-l'];
                options.interface = inter;
                scan(function (err, data) {
                    if (err) {
                        // console.log('Error in ' + options.interface + ' interface. Error code is ' + err);
                    } else {
                        if (options.listByInterface) {
                            var result = {};
                            result[inter] = data;
                            list.push(result);
                        } else {
                            if (data && data.length) {
                                for (var kk = 0; kk < data.length; kk++) {
                                    list.push(data[kk]);
                                }
                               }
                        }
                    }
                    cbl();
                }, options);
            } else {
                cbl();
            }
        }, function (err) {
            callback(err, list)
        });
    }
}

module.exports = scanner;
module.exports.promisify = function () {
    return require('./arpscanner-promise')
};
// exports.DEFAULTS = DEFAULTS;
